//////////////////////////////////////////////////////////////////////////////
//
//  Copyright (C) 2007-2011 Infinite Z, Inc.  All Rights Reserved.
//
//  File:       HeadTrackingGlutWindow.cpp
//  Content:    Sample code demonstrating use of 
//				      ZSStereo's Head Tracking in Default (simple) mode.
//
//////////////////////////////////////////////////////////////////////////////

#ifdef _WIN32
#include <windows.h>
#endif

#include "HeadTrackingGlutWindow.h"
#include "Scene.h"
#include "ZSpaceInterface.h"

//C includes after the C++ includes, else err C2766 regarding the stdbool.h
#include "../../npdata.h"
#include "../../npctrl.h"
extern "C" 
{
	#include "../npgl.h"
	#include "../npmouse.h"
}

using namespace zspace;
using namespace zspace::stereo;

char const * const HeadTrackingGlutWindow::WINDOW_NAME = "ANTz";			//S3D

HeadTrackingGlutWindow* HeadTrackingGlutWindow::s_instance = NULL;

HeadTrackingGlutWindow::HeadTrackingGlutWindow()
: m_scene(NULL), m_zSpaceInterface(NULL)
{
}

HeadTrackingGlutWindow*
HeadTrackingGlutWindow::GetInstance()
{
  if(s_instance == NULL)
    s_instance = new HeadTrackingGlutWindow;
  return s_instance;
}

HeadTrackingGlutWindow::~HeadTrackingGlutWindow()
{
  delete m_zSpaceInterface;
}

static GLfloat DIFFUSE_LIGHT[]   = {0.4f, 0.4f, 0.4f, 1.0f};	 /* diffuse light. */
static GLfloat LIGHT_POSITION[]  = {0.2f, 0.2f, 0.6f, 1.0}; 
static GLfloat LMODEL_AMBIENT[]  = {0.5, 0.5, 0.5, 1.0}; 

int
HeadTrackingGlutWindow::initialize(int windowX, int windowY, int windowWidth, int windowHeight)
{
//  glutInitWindowSize (windowWidth, windowHeight);								//S3D
//  glutInitDisplayMode ( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH | GLUT_STEREO);
//  glutCreateWindow (WINDOW_NAME);

  // Register callbacks
  RegisterCallbacks();
/*																				//S3D
  // Enable depth testing
  glEnable(GL_DEPTH_TEST);
  // Enable lighting
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glShadeModel(GL_SMOOTH);

  // Enable a single OpenGL light.
  glLightfv(GL_LIGHT0, GL_POSITION, LIGHT_POSITION);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, DIFFUSE_LIGHT);
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, LMODEL_AMBIENT);
*/
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(35.0f, ((float)windowWidth)/windowHeight, 1.0f, 2000.0f);
//  gluPerspective(45, ((float)windowWidth)/windowHeight, .1f, 10000.0f);	//S3D modify?
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  return true;
}


void HeadTrackingGlutWindow::Clean()
{
  if(s_instance != NULL)
  {
    if(s_instance->m_zSpaceInterface != NULL)
    {
      s_instance->m_zSpaceInterface->clean();
      delete s_instance->m_zSpaceInterface;
      s_instance->m_zSpaceInterface = NULL;
    }
    delete s_instance;
  }
}

/////////////////////////////////////
// GLUT callback system

void
HeadTrackingGlutWindow::RegisterCallbacks()
{
  ::glutDisplayFunc (GlutDisplayFunc);
  ::glutReshapeFunc (GlutReshapeFunc);
 // ::glutKeyboardFunc (GlutKeyboardFunc);
  ::glutIdleFunc (GlutIdleFunc);
}

void
HeadTrackingGlutWindow::GlutDisplayFunc()
{
  GetInstance()->glutDisplayFunc();
}
void
HeadTrackingGlutWindow::GlutKeyboardFunc(unsigned char key, int x, int y)
{
  GetInstance()->glutKeyboardFunc(key, x, y);
}
void
HeadTrackingGlutWindow::GlutReshapeFunc(GLint width, GLint height)
{
  GetInstance()->glutReshapeFunc(width, height);
}
void
HeadTrackingGlutWindow::GlutIdleFunc()
{
  GetInstance()->glutIdleFunc();
}

void
HeadTrackingGlutWindow::glutDisplayFunc()
{
  // Render current scene frame
  renderFrame();
}

void
HeadTrackingGlutWindow::glutKeyboardFunc(unsigned char key, int x, int y)
{
  switch (key)
  {
    case 27:             // ESCAPE key
      exit (0);
      break;
    case 'S':
    case 's':
      m_zSpaceInterface->onSwapRenderTarget();						//S3D add feature
      break;
    case 'T':
    case 't':
      m_zSpaceInterface->toggleHeadTracking();
      break;
  }
}

void
HeadTrackingGlutWindow::glutReshapeFunc(int width, int height)
{
//zziz		pData data = (pData) npGetDataRef();

//zziz		data->io.gl.width = width;
//zziz		data->io.gl.height = height;

  m_zSpaceInterface->onResize(width, height);						//S3D wrap this
//zziz	  glMatrixMode(GL_PROJECTION);
//zziz	  glLoadIdentity();
//zziz	  gluPerspective(35.0f, ((float)width)/height, 1.0f, 2000.0f);
//zziz	  glViewport(0, 0, width, height);
 // renderFrame(); //S3D
  // Force redraw
//  glutPostRedisplay();	//S3D

 // 	renderFrame();

  npGLResizeScene (width, height);	//zziz
}

void
HeadTrackingGlutWindow::glutIdleFunc()
{
  m_zSpaceInterface->captureTargets();						//S3D wrap this
  if(m_scene != NULL)
    m_scene->update();
  
  // HACK: no callback for window move, will need to poll it every frame
  int oldx = getWindowX();
  int x = glutGet(GLUT_WINDOW_X);
  int y = glutGet(GLUT_WINDOW_Y);
  if( (x != getWindowX()) && (y != getWindowY()) )
    moveFunc(x, y);											//S3D wrap this

 // npGlutDrawGLSceneIZ();		//S3D

  npLookAt();												//S3D

  // Force redraw
  glutPostRedisplay();
}

void HeadTrackingGlutWindow::moveFunc(int x, int y)
{
  m_zSpaceInterface->onMove(x, y);							//S3D wrap this
}

// GLUT callback system
/////////////////////////////////////

/////////////////////////////////////
// Display rendering functionality

void
HeadTrackingGlutWindow::getDefaultCameraPosition(float* pos3look3up3)
{
  if(!m_zSpaceInterface->getDefaultCameraPosition(pos3look3up3))
  {
    pos3look3up3[0] = 0.0f;
    pos3look3up3[1] = 0.0f;
    pos3look3up3[2] = 20.0f;		//S3D	was 1.0
    pos3look3up3[3] = 0.0f;
    pos3look3up3[4] = 0.0f;
    pos3look3up3[5] = 0.0f;
    pos3look3up3[6] = 0.0f;
    pos3look3up3[7] = 1.0f;
    pos3look3up3[8] = 0.0f;
  }
}

void HeadTrackingGlutWindow::renderFrame()					//S3D need to modify
{
	static int firstTime = 1; //zziz
	int period = 16; //frame period in ms (ie 60fps is 16.66667... approx 16ms
	static long int previousTime = 0;
	long int time = 0;
	long int deltaTime = 0;

	pData data = (pData) npGetDataRef();	//zziz

	//init time on first run
	if (previousTime == 0)
		previousTime = glutGet(GLUT_ELAPSED_TIME);


//	if (deltaTime < period)
//		nposSleep( period - deltaTime);			//calls OS specific command

  if(!m_scene)
    return;

//  npLookAt();												//S3D
	npUpdateCtrl ( npGetDataRef() );
 
  // Render each eye view.
  renderEye(StereoFrustum::STEREO_EYE_LEFT);
	 npDrawHUDIZ();	//zziz
	 if (data->io.gl.pickPass)	//zziz
	 {
		//set readbuffer
		//pick which buffer to read from
		if (data->io.gl.swapEyes)//toggle)
			glReadBuffer(GL_BACK_RIGHT);
		else
			glReadBuffer(GL_BACK_LEFT);

		npPickPostIZ();

		//now render for visible, may need to clear buffer again
		glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	//ask iz? perhaps already done
		renderEye(StereoFrustum::STEREO_EYE_LEFT);
		npDrawHUDIZ();

		//try other eye if result id=0
		if (data->io.gl.pickID == 0)
		{
			data->io.gl.pickPass = true;

			renderEye(StereoFrustum::STEREO_EYE_RIGHT);
			npDrawHUDIZ();

			if (data->io.gl.swapEyes)//toggle)
				glReadBuffer(GL_BACK_LEFT);
			else
				glReadBuffer(GL_BACK_RIGHT);

			
			npPickPostIZ();

			glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	//ask iz? perhaps already done
		}
	//	npPickMousePostProcess(data);
	 }	//zziz

  renderEye(StereoFrustum::STEREO_EYE_RIGHT);			//S3D reversed for pickPass
	npDrawHUDIZ();	//zziz

  // Inform the window's stereo buffer that rendering is complete.
  m_zSpaceInterface->endStereo();

	time = glutGet(GLUT_ELAPSED_TIME);
	deltaTime = time - previousTime;
	//	printf("deltaTime: %d \n", deltaTime);

  if (deltaTime < period)
	  Sleep( period - deltaTime);			//S3D replaced 16 with this

  // Show the back buffer
  npTimingIZ();
  glutSwapBuffers();

  if (firstTime)	//zziz
  {
		firstTime = 0;
		npCtrlCommand(kNPcmdFullscreen, npGetDataRef());
  }					//zziz
}

void HeadTrackingGlutWindow::renderEye(StereoFrustum::StereoEye eye)
{
  m_zSpaceInterface->setupStereoEye(eye);

  renderScene();

  // restore both matrix stacs
  m_zSpaceInterface->clearStereoEye();
}

void
HeadTrackingGlutWindow::renderScene()
{
// m_scene->render(); //Draw object centered at (0, 0, 0)		//draw antz scene

	npDrawNodesIZ ();				//draws mesh, graphs, pins, etc...			//S3D

	if	(npSwapEyesIZ())
		m_zSpaceInterface->onSwapRenderTarget();
}

// Display rendering functionality
/////////////////////////////////////

/////////////////////////////////////
// ZSpace additions

int HeadTrackingGlutWindow::initializeEyes()
{
  // initializes the concept of OpenGl stereo, specifically
  //  synchronizing left and right eyes
  // should only have to happen once
#ifdef _WIN32
  // initialize/synchronize only works on Windows right now
  HWND hWnd = FindWindow(NULL, WINDOW_NAME);
  return m_zSpaceInterface->InitializeEyes(hWnd);
#else
  return true;
#endif
}

int HeadTrackingGlutWindow::initializeStereo(int windowX, int windowY, int windowWidth, int windowHeight)
{
  // Set up and initialize stereo windows
  m_zSpaceInterface = new ZSpaceInterface();
  // Initialize the stereo window and viewport.
  if(!m_zSpaceInterface->initialize(windowX, windowY, windowWidth, windowHeight))
  {
    ZS_LOG(ERR, "Failed to initialize the Stereo window.\n");
    delete m_zSpaceInterface;
    return false;
  }
  return true;
}

// ZSpace additions
/////////////////////////////////////


/////////////////////////////////////
// miscellaneous window management

int
HeadTrackingGlutWindow::getWindowX() const
{
  return m_zSpaceInterface->getWindowX();
}

int
HeadTrackingGlutWindow::getWindowY() const
{
  return m_zSpaceInterface->getWindowY();
}

int
HeadTrackingGlutWindow::getWindowWidth() const
{
  return m_zSpaceInterface->getWindowWidth();
}

int
HeadTrackingGlutWindow::getWindowHeight() const
{
  return m_zSpaceInterface->getWindowHeight();
}


