//////////////////////////////////////////////////////////////////////////////
//
//  Copyright (C) 2007-2011 Infinite Z, Inc.  All Rights Reserved.
//
//  File:       HeadTrackingScene.h
//  Content:    Sample code demonstrating use of 
//				      ZSStereo's Head Tracking in Default (simple) mode.
//
//////////////////////////////////////////////////////////////////////////////

#ifndef __ZSPACE_SAMPLES_HEAD_TRACKING_SCENE_H__
#define __ZSPACE_SAMPLES_HEAD_TRACKING_SCENE_H__

#include <windows.h>
#include <string>
#include <GL\gl.h>
#include <GL\glu.h>

// Default to windowed mode
#define ZSSAMPLE_FULLSCREEN 0								//S3D

#include "ZSpace/Stereo/Gl/StereoGl.h"
#include "ZSpace/Tracker/TrackerSystem.h"

// Define the screen resolution for windowed mode
#if ZSSAMPLE_FULLSCREEN == 0
#define WINDOW_WIDTH	1024								//S3D  1920x1080
#define WINDOW_HEIGHT	768
#endif

#define CUBE_WIDTH 0.06f

class HeadTrackingScene
{
public:
	HeadTrackingScene(int windowX, int windowY, int windowWidth, int windowHeight);

	~HeadTrackingScene();

  // Initialize the scene.
  bool initialize(HWND hWnd);

  // Clean up resources.
  void clean();

  // Single frame rendering function.
  void renderFrame();

  void onMove(int x, int y);
  void onResize(int width, int height);
  void onSwapRenderTarget();

  void toggleHeadTracking();

private:
  HDC						m_hDc;          // Handle for the device context.
  HGLRC						m_hGlrc;        // Handle for OpenGl rendering context.
  HWND						m_hWnd;         // Handle for the window.
  GLuint					m_textureName;  // GL texture name.

  int                     m_windowX;      // Horizontal X position of the window's top left corner.
  int                     m_windowY;      // Vertical Y position of the window's top left corner.
  int                     m_windowWidth;  // The window's width.
  int                     m_windowHeight; // The window's height.

  // Stereo members.
  zspace::stereo::StereoViewport*   m_stereoViewport;
  zspace::stereo::StereoWindow*     m_stereoWindow;

  // <SampleCode1>

  // Tracking members.
  zspace::tracker::TrackerSystem*   m_trackerSystem;
  zspace::tracker::TrackerTarget*   m_headTarget;
  bool m_isHeardTrackingEnabled;
  // </SampleCode1>

private:
  // Initialize the OpenGl rendering context.
  void initializeGl(HWND hWnd);

  // Initialize the stereo window and viewport.
  void initializeStereo();

  // Load 3D models into the video RAM.
  void initializeScene();

  // BMP texture loader
  bool createTextureFromBmp(char* filename);

  // Set up modelview matrix, projection matrix, and render buffer for specified eye
  void renderEye(zspace::stereo::StereoFrustum::StereoEye eye);

  // Renders scene objects.
  void renderScene();
};

//-----------------------------------------------------------------------------
int npInitIZ(); 

#endif // __ZSPACE_SAMPLES_HEAD_TRACKING_SCENE_H__
